@@ -4,7 +4,7 @@ package Locales;
 # use strict;
 # use warnings;
 
-$Locales::VERSION      = '0.23';    # change in POD
+$Locales::VERSION      = '0.26';    # change in POD
 $Locales::cldr_version = '2.0';     # change in POD
 
 #### class methods ####
@@ -272,7 +272,6 @@ sub get_plural_form {
     #     and also perl's modulus-on-negative-values behavior
     my $abs_n = abs($n);    # negatives keep same category as positive
 
-    # As of 0.22 this will be skipped for mosules included w/ the main dist
     if ( !$self->{'language_data'}{'misc_info'}{'plural_forms'}{'category_rules_function'} ) {
         $self->{'language_data'}{'misc_info'}{'plural_forms'}{'category_rules_function'} = Locales::plural_rule_hashref_to_code( $self->{'language_data'}{'misc_info'}{'plural_forms'} );
         if ( !defined $self->{'language_data'}{'misc_info'}{'plural_forms'}{'category_rules_function'} ) {
@@ -777,7 +776,7 @@ sub normalize_tag {
 }
 
 sub normalize_tag_for_datetime_locale {
-    my ( $pre, $pst ) = split_tag( $_[0] );
+    my ( $pre, $pst ) = split_tag( $_[0] ); # we only do language[_territory]
     return if !defined $pre;
 
     if ($pst) {
@@ -788,6 +787,18 @@ sub normalize_tag_for_datetime_locale {
     }
 }
 
+sub normalize_tag_for_ietf {
+    my ( $pre, $pst ) = split_tag( $_[0] ); # we only do language[_territory]
+    return if !defined $pre;
+
+    if ($pst) {
+        return $pre . '-' . uc($pst);
+    }
+    else {
+        return $pre;
+    }
+}
+
 sub normalize_for_key_lookup {
     my $key = $_[0];
     return if !defined $key;
@@ -805,6 +816,8 @@ sub plural_rule_string_to_javascript_code {
     my $perl = plural_rule_string_to_code( $plural_rule_string, $return );
     $perl =~ s/sub { /function (n) {/;
     $perl =~ s/\$_\[0\]/n/g;
+    $perl =~ s/ \(n \% ([0-9]+)\) \+ \(n-int\(n\)\) /n % $1/g;
+    $perl =~ s/int\(/parseInt\(/g;
     return $perl;
 }
 
@@ -818,7 +831,10 @@ sub plural_rule_string_to_code {
 
     my %m;
     while ( $plural_rule_string =~ m/mod ([0-9]+)/g ) {
-        $m{$1} = "(\$_[0] % $1)";
+
+        # CLDR plural rules (http://unicode.org/reports/tr35/#Language_Plural_Rules):
+        #      'mod' (modulus) is a remainder operation as defined in Java; for example, the result of "4.3 mod 3" is 1.3.
+        $m{$1} = "( (\$_[0] % $1) + (\$_[0]-int(\$_[0])) )";
     }
 
     my $perl_code = "sub { if (";
@@ -907,7 +923,7 @@ sub plural_rule_hashref_to_code {
                 # Does $n match $hr->{$cat} ?
 
                 if ( ref( $hr->{'category_rules_compiled'}{$cat} ) ne 'CODE' ) {
-                    $hr->{'category_rules_compiled'}{$cat} = eval "$hr->{'category_rules_compiled'}{$cat}";
+                    $hr->{'category_rules_compiled'}{$cat} = eval "$hr->{'category_rules_compiled'}{$cat}";    # As of 0.22 this will be skipped for modules included w/ the main dist
                 }
 
                 if ( $hr->{'category_rules_compiled'}{$cat}->($n) ) {
@@ -931,6 +947,32 @@ sub get_cldr_plural_category_list {
     return qw(one two few many other zero);             # quant() arg order
 }
 
+sub get_fallback_list {
+    my ( $self, $special_lookup ) = @_;
+
+    my ( $super, $ter ) = split_tag( $self->{'locale'} );
+    return (
+        $self->{'locale'},
+        ( $super ne $self->{'locale'} && $super ne 'i' ? $super : () ),
+        ( @{ $self->{'language_data'}{'misc_info'}{'fallback'} } ),
+        (
+            defined $special_lookup && ref($special_lookup) eq 'CODE'
+            ? ( map { my $n = Locales::normalize_tag($_); $n ? ($n) : () } $special_lookup->( $self->{'locale'} ) )
+            : ()
+        ),
+        'en'
+    );
+}
+
+# get_cldr_$chart_$type_$name or better naming ?
+sub get_cldr_number_symbol_decimal {
+    return $_[0]->{'language_data'}{'misc_info'}{'cldr_formats'}{'_decimal_format_decimal'} || '.';
+}
+
+sub get_cldr_number_symbol_group {
+    return $_[0]->{'language_data'}{'misc_info'}{'cldr_formats'}{'_decimal_format_group'} || ',';
+}
+
 1;
 
 __END__
@@ -943,7 +985,7 @@ Locales - Methods for getting localized CLDR language/territory names (and a sub
 
 =head1 VERSION
 
-This document describes Locales version 0.23
+This document describes Locales version 0.26
 
 =head1 SYNOPSIS
 
@@ -1139,6 +1181,34 @@ See L<http://unicode.org/repos/cldr-tmp/trunk/diff/by_type/names.localeDisplayPa
 
 Same as get_locale_display_pattern_from_code() except it should use less-overhead. Can be called as a function also so you can use it without creating an object.
 
+=item get_cldr_number_symbol_decimal()
+
+Returns the decimal point symbol for the object's locale. Takes no arguments.
+
+For formatting numbers use get_formatted_decimal().
+
+=item get_cldr_number_symbol_group() 
+
+Returns the integer grouping symbol for the object's locale. Takes no arguments.
+
+For formatting numbers use get_formatted_decimal().
+
+=item get_fallback_list()
+
+Returns a fallback list of locales in the order they apply based on the object's locale.
+
+The basic list will be: object's locale, object's super if any, the object's CLDR fallback if any, “special lookup” if any, 'en'
+
+    my @list = $fr_ca->get_fallback_list();
+    # fr_ca fr en
+
+"special lookup" is a code ref that can be passed in as the first optional arg. 
+
+It is given the object's locale when called and should return a list of locale tags (they will be normalized).
+
+    my @list = $fr_ca->get_fallback_list(sub { return $_[0] =~ m/fr/ ? qw(i_yoda i_love_rhi) : () } );
+    # fr_ca fr i_yoda i_love_rhi en
+
 =item get_plural_form()
 
 Takes a number and returns the plural category that the number fits under for the object's locale.
@@ -1291,10 +1361,18 @@ Returns the normalized tag.
 
 =item Locales::normalize_tag_for_datetime_locale()
 
-Like normalize_tag() except the return value should be suitable for L<DataTime::Locale>
+Like normalize_tag() except the return value should be suitable for L<DateTime::Locale>
 
    print Locales::normalize_tag_for_datetime_locale("  en-GB\n "); # 'en_GB'
 
+=item Locales::normalize_tag_for_ietf()
+
+Like normalize_tag() except the return value should be suitable for IETF.
+
+This is not a comprehensive IETF formatter, it is intended (for now at least) for the subset of tags Locales.pm uses.
+
+   print Locales::normalize_tag_for_ietf("  en_gb\n "); # 'en-GB'
+
 =item Locales::split_tag()
 
 Takes a single argument, the locale tag to split into language and territory parts.
@@ -1379,7 +1457,7 @@ This takes a hashref that contains rules, puts them in the hash, and returns an
 
 Same as Locales::plural_rule_string_to_code() except it returns javascript code instead of perl code.
 
-Used internally when building this distributions share/javascript/ contents (JSON files for 'misc_info' hash and 'code_to_name').
+Used internally when building this distribution's share/misc_info contents.
 
 =back
 
@@ -1408,7 +1486,7 @@ None reported.
   - generally improve get_code_from_* lookups
   - tests that misc info doesn't get odd structs from XML instead of a string
   - ? install share/ via L<File::ShareDir> mechanism ?
-  - vet share/javascript/ && document better
+  - vet share/ && document better
 
 =head1 DEPRECATED MODULES/INTERFACE
 
